########################################################################################
###
### SUPPORTING MATERIAL FOR THE ARTICLE
### Baraldo S., Ieva F., Paganoni A. M., Vitelli V., "Outcome prediction for heart 
### failure telemonitoring via generalized linear models with functional covariates", 
### Scandinavian Journal of Statistics (2012)
###
###	Author: Stefano Baraldo
###	Date:   7 June 2012
### 
###	Test study on artificial dataset, based on the application and methods presented
### in the referenced work
###
########################################################################################

### Pagkages loading
library(gcmrec)   # Package for counting process model estimation
library(cobs)     # Package for constrained L1 B-splines

### Utility functions used throughout the script
source("GLMHF_funcs_Supp.R")

### Public Health Database-like data
load(file="PHD_data_Supp.RData")
head(PHD_data)
# Data preformatted for use with function gcmrec of omonymous package. Each line
# represents data on a single hospital admission for a subject
# -     id:   subject index
# - age_in:   subject age at 1 January 2004
# -    age:   subject age at event
# -   time:   waiting times from preceding hospitalization or beginning of study. Last
#             line corresponds to waiting time between last admission and censoring time.
# - event:    1 if the event is a hospitalization, 0 if censoring
# -    Nm:    N_i(t^-) for subject i

### Telemonitoring-like data
load(file="survey_data_Supp.RData")
head(survey_data)
# Each line represents data available for a subject that underwent telemonitoring
# -         id:   subject index
# -        sex:   subject gender
# -  diagnosis:   main diagnosis for last admission before telemonitoring
# -   etiology:   etiology for last admission before telemonitoring
# - conclusion:   1 the conclusion is regular, 0 for early interruption


########################################################################################
### Phase 1: estimation of cumulative hazard processes and dimensional reduction
########################################################################################


### HAZARD MODEL ESTIMATION ###

# Event and censoring times
graph.caltimes(PHD_data)

tmax <- 2000              #Maximum global observation time
n <- dim(survey_data)[1]  # Sample size

### Cox-type model
count_model = gcmrec(Survr(id,time,event)~age, data=PHD_data, s=tmax, Frailty=FALSE, typeEffage= "minimal", rhoFunc="alpha to k")
count_model
plot(count_model$diseff,count_model$Lambda,xlab="time",ylab=expression(hat(Lambda)[0](t)),type="s")

### Smoothing of Lambda_0(t).

t <- count_model$diseff
Lambda0 <- count_model$Lambda
# Smooth version of Lambda0
Lambda0S <- cobs(t,Lambda0,constraint=c("increase"),pointwise=matrix(c(0,0,0),nrow=1),nknots=20,lambda=0,toler.kn=0)
# Comparison between basic and smooth estimate
plot(Lambda0S$x,Lambda0S$fitted,type="l",main="",ylab="Baseline cumulative hazard",xlab="time")
points(t,Lambda0,type="s",lty=2)

### RECONSTRUCTION OF CUMULATIVE HAZARD PROCESSES ###

# For ease of use, we create some R functions that return evaluations of mathematical 
# functions in vectors of time points

# Lambda_0
#      t  = evaluation points
Lambda0_fun <- function(t){
  return(basis(t, Lambda0S$knots) %*% Lambda0S$coef)
}

# Subject age
#      t  = evaluation points
#     t0  = age at the beginning of observation time (1 January 2004)
Xage <- function(t,t0){
  return(t0+t/365)
}

# First derivative of age
dXage <- 1/365

# Since event times are integer (expressed in days), it is convenient to
# compute reconstructed cumulative hazard functions on a daily spaced grid
times <- seq(0,max(t),by=1)
nt <- length(times)

# Estimated model parameters
alpha <- count_model$coef[1]
beta <- count_model$coef[-1]
p <- length(beta)

# Defining
#    P_i(t) = Lambda_0(t) exp(beta^T X_i(t)) - int_0^t( Lambda_0(s) exp(beta^T X_i(t)) beta^T X_i(t)' ds )
# we have
#    Lambda_i(t) = sum_(k=0)^(N_i(t^-)) exp(k log(alpha)) (P_i(min{t_(k+1)^i,t}) - P_i(t_k^i)).
# One subject for each column.
Pt <- matrix(0, nrow=nt, ncol=n)
for(i in 1:n){
  t0 <- PHD_data$age_in[which(PHD_data$id==i)[1]]
  integrand <- function(times){
    Lambda0_fun(times)*exp(beta[1]*Xage(times,t0))*beta[1]*dXage
  }
  Pt[,i] <- Lambda0_fun(times)*exp(beta[1]*Xage(times,t0)) - int_fun(integrand,times)
}

# Nt^- evaluated on the discrete grid (variable "times"). One subject for each column
Ntm <- matrix(0, nrow=nt, ncol=n)
for(i in 1:n){
  idx <- which(PHD_data$id==i)
  jtimes <- cumsum(PHD_data$time[idx])
  lj <- length(jtimes)
  Ntm[which(times > jtimes[lj]),i] <- NA
  if(lj>1){
    for(j in 1:(lj-1)){
      Ntm[,i] <- Ntm[,i] + (times > jtimes[j])
    }
  }
}

# Reconstruction of individual cumulative hazard functions, evaluated at discrete times
LambdaEst <- matrix(0, nrow=nt, ncol=n)
for(i in 1:n){
  LambdaEst[,i] <- cumsum((alpha^Ntm[,i])*c(0,diff(Pt[,i])))
}

# For plotting purposes
timesmat <- NULL
for(i in 1:n){
  timesmat <- cbind(timesmat,times)
}

# Plot of reconstructed realizations of cumulative hazard processes
matplot(timesmat,LambdaEst,type="l",xlab="time",main=expression(Lambda[i](t)))

### SUBJECT SELECTION ###

# Only subjects with at least 2 years (730 days) observed are considered
clengths <- NULL
for(i in 1:(dim(LambdaEst)[2])){
  clengths <- c(clengths,length(which(!is.na(LambdaEst[,i]))))
}
idx730 <- which(clengths>=731)
LamPRE <- LambdaEst[,idx730]
timesPRE <- timesmat[,idx730]
clengths <- clengths[idx730]
# For them, only the hazard over the last 2 years before telemonitoring(730 days) 
# is considered
for(i in 1:(dim(LamPRE)[2])){
  LamPRE[1:731,i] <- LamPRE[(clengths[i]-730):clengths[i],i]
}
LamPRE <- LamPRE[1:731,]
timesPRE <- timesPRE[1:731,]
colnames(LamPRE) <- idx730

### FPCA ###

nc <- dim(LamPRE)[2]
m <- dim(timesPRE)[1]
h <- (timesPRE[m,1]-timesPRE[1,1])/(m-1)

# Functional data considered in FPCA.
matplot(timesPRE,LamPRE,type="l",xlab="time (days)",ylab="cumulative hazard",main="Reconstructed hazard functions")
mu <- apply(LamPRE,MARGIN=1,"mean")
points(timesPRE[,1], mu ,type="l", lwd=3)
# Since subjects and time intervals are selected so that the problem of censoring is avoided,
# the pointwise mean is a good estimator of the mean process. For in case of censored
# curves, see the function increMean in GLMHF_funcs.R

# Eigenvalues and eigenfunctions
pca <- prcomp(t(LamPRE))
efuncs <- pca$rotation*sqrt(1/h)
evalues <- pca$sdev^2*h

# Principal components diagnostics

# Eigenvalues
sum.evalues <- sum(evalues)
percent.var <- evalues/sum.evalues
# Choose the minimum K1 such that the sum of eigenvalues (variances) is at least 99% of total
K <- min(which(cumsum(percent.var)>=0.99))
# Summary table
evtable <- rbind(evalues[1:K], 100*evalues[1:K]/sum.evalues, 100*cumsum(evalues[1:K])/sum.evalues)
rownames(evtable) <- c("eigenvalue", "% variance", "cumulated % variance")
colnames(evtable) <- paste("component",1:K)
evtable

# Scree plots:
par(mfrow=c(1,2))
plot(evalues[1:K], xlab="", ylab="", main="eigenvalues" ,type="b")
plot(evtable[3,],xlab="",ylab="",main="% of variance",type="b",ylim=c(0,100))
abline(99,0,lty=2)

# Eigenfunctions
# Sign chosen to have positive integral, for ease of interpretation
for(i in 1:K) {
  tempfun <- efuncs[,i]
  tempsign <- sum(tempfun)
  efuncs[,i] <- ifelse(tempsign<0, -1,1) * tempfun
}

# Plot of first k eigenfunctions
k=K
par(mfcol=c(2,k),mar=c(2.5,2,2.5,1))
ylim <- range(efuncs[,1:k])
ylim2 <- c(0,5.5)
for(i in 1:k) {
  v1 <- efuncs[,i]
  plot(timesPRE[,1], v1, type="l", ylab="", xlab="time (days)", ylim=ylim, lwd=2, main=paste("Component ", i,", ",100*round(percent.var[i],3), "%", sep=""))
  text(2,-0.95, "time (days)")
  v2 <- v1 * sqrt(evalues[i])
  widep <- ((1:m)%%30)==0
  plot(timesPRE[,1], mu, type="l", ylim=ylim2, lwd=3)
  points(timesPRE[widep,1], mu[widep] + v2[widep], lwd=3, pch="+")
  points(timesPRE[widep,1], mu[widep] - v2[widep], lwd=3, pch="-")
}

# Scores on the first K principal components
score <- NULL
for(i in 1:K){
  score <- cbind(score,(as.matrix(t(LamPRE-mu)) %*% cbind(efuncs[,i]))*h)
}

# Projection on first 2 P.C.
score1 <- score[,1]
score2 <- score[,2]
proj1 <- apply(X=cbind(score1),FUN=`*`,MARGIN=1,y=efuncs[,1]) + apply(X=cbind(score2),FUN=`*`,MARGIN=1,y=efuncs[,2])+mu
layout(cbind(1,2))
ylim=range(c(range(LamPRE),range(proj1)))
matplot(timesPRE, proj1, type="l",xlab="time (days)", ylab="cumulative hazard", main=paste("projection of cumulative hazard on first p.c.", sep=" "), ylim=ylim)
matplot(timesPRE, LamPRE, type="l", xlab="time (days)", ylab="cumulative hazard", main="original dataset", ylim=ylim)



########################################################################################
### Phase 2: logistic regression for outcome prediction
########################################################################################

# Patient with at least 730 days observed extracted from the clinical survey dataset
survey_sub <- data.frame(survey_data[idx730,], score1, score2)

# Logistic regression model. The response "conclusion" equals 1 if the telemonitoring period
# has been completed (~180 days), 0 if it has been interrupted.
logit_conc <- glm(conclusion~score1+score2+sex+diagnosis+etiology, data=survey_sub, family=binomial(link="logit"))
summary(logit_conc)

# Some indices for model evaluation and comparison

# Briar score
briar <- mean((logit_conc$fitted.values-logit_conc$y)^2)
briar
# AIC
logit_conc$aic
# K-fold cross-validation error
# Selection of subjects without missing values in used covariates
nonmiss <- which(!is.na(survey_sub$conclusion) & !is.na(survey_sub$sex) & !is.na(survey_sub$diagnosis) & !is.na(survey_sub$etiology))
kcv <- 10
cverr <- cv.glm(data=survey_sub[nonmiss,], glmfit=logit_conc, K=kcv)
cverr$delta # raw and adjusted